pub struct Multicast { /* private fields */ }Expand description
UDP multicast adapter for LAN peer discovery and sync.
Broadcasts Gun protocol messages to the multicast group 233.255.255.255:7654
and receives messages from other peers on the same LAN.
Implementations